'use strict';

var fetchres = require('fetchres');
var logger = require('ft-next-logger');
var eagerFetch = require('./utils/eager-fetch');

function formatSearchTerm(term) {
	var query = term.split(':');

	return Object.defineProperty({}, 'item.metadata.' + query[0] + '.term.name', {
		value: query[1],
		enumerable: true
	});
}

function formatDateRange(from, to) {
	var dateFrom = new Date();
	var dateTo = new Date();

	dateFrom.setDate(dateFrom.getDate() - from);
	dateTo.setDate(dateTo.getDate() - to);

	return {
		gte: dateFrom.toISOString(),
		lt: dateTo.toISOString()
	};
}

function formatAggregations(taxonomies, minCount) {
	return taxonomies.reduce(function(previous, taxonomy) {
		return Object.defineProperty(previous, taxonomy, {
			enumerable: true,
			value: {
				terms: {
					field: 'item.metadata.' + taxonomy + '.term.name',
					min_doc_count: minCount
				}
			}
		});
	}, {});
}

function formatPostBody(options) {
	return {
		query: {
			bool: {
				must: [
					{
						match: formatSearchTerm(options.query)
					},
					{
						range: {
							'item.lifecycle.initialPublishDateTime': formatDateRange(options.range, 1)
						}
					}
				]
			}
		},

		sort: {
			'item.lifecycle.initialPublishDateTime': {
				order: 'desc'
			}
		},

		aggs: formatAggregations(options.taxonomies, 2)
	};
}

function pickAndFlattenAggregations(data, options) {
	var aggregation = [];

	options.taxonomies.forEach(function(taxonomy) {
		if (!data.aggregations[taxonomy]) return;

		data.aggregations[taxonomy].buckets.forEach(function(item) {
			aggregation.push( Object.defineProperty(item, 'taxonomy', {
				value: taxonomy
			}) );
		});
	});

	return aggregation;
}

function sortAggregation(aggregation) {
	return aggregation.sort(function(a, b){
		if (a.doc_count > b.doc_count) {
			return -1;
		}

		if (a.doc_count < b.doc_count) {
			return 1;
		}

		return 0;
	});
}

module.exports = function(options) {
	var postBody = formatPostBody(options);

	return eagerFetch('https://' + process.env.ELASTIC_SEARCH_HOST + '/v1_api_v2/item/_search?search_type=count', {
		body: JSON.stringify(postBody),
		method: 'POST',
		timeout: 3000
	})
		.then(function(response) {
			if (!response.ok) {
				logger.warn('Failed getting related tags for', options);
			}

			return response;
		})
		.then(fetchres.json)
		.then(function(data) {
			if (!data.aggregations) return [];

			return sortAggregation(pickAndFlattenAggregations(data, options));
		});
};
